@@ -1,11 +1,13 @@
 /*
- * Copyright 2005 Fluendo S.L.
+ * Copyright <2005, 2006, 2007, 2008> Fluendo S.A.
  */
  
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include "../common/gst-fluendo.h"
+
 #include "flump3dec.h"
 
 GST_DEBUG_CATEGORY (flump3debug);
@@ -103,22 +105,24 @@ flump3dec_class_init (FluMp3Dec *klass)
 static void
 flump3dec_base_init (FluMp3DecClass *klass)
 {
+#ifdef USE_IPP
+  #define LONGNAME "Fluendo MP3 Decoder (IPP build)"
+#else
+#ifdef USE_LIBOIL
+  #define LONGNAME "Fluendo MP3 Decoder (liboil build)"
+#else
+  #define LONGNAME "Fluendo MP3 Decoder (C build)"
+#endif
+#endif
   /* FIXME: Which email address to put? */
   static GstElementDetails details = 
     GST_ELEMENT_DETAILS (
-#ifdef USE_IPP
-      "Fluendo MP3 Decoder (IPP build)",
-#else
-#ifdef USE_LIBOIL
-      "Fluendo MP3 Decoder (liboil build)",
-#else
-      "Fluendo MP3 Decoder (C build)",
-#endif
-#endif
+	  LONGNAME,
       "Codec/Decoder/Audio",
       "Decodes MPEG-1 Layer 1, 2 and 3 streams to raw audio frames",
       "Fluendo Support <support@fluendo.com>"
     );
+#undef LONGNAME
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   gst_element_class_add_pad_template (element_class,
@@ -201,25 +205,42 @@ flump3dec_flush (FluMp3Dec *flump3dec)
     gst_buffer_unref (flump3dec->pending_frame);
     flump3dec->pending_frame = NULL;
   }
+  
+  flump3dec->need_discont = TRUE;
 }
 
 /* Decode a buffer */
 static GstFlowReturn
 flump3dec_sink_chain (GstPad *pad, GstBuffer *buf)
 {
-  FluMp3Dec        *flump3dec = FLUMP3DEC (GST_PAD_PARENT (pad));
-  GstClockTime    new_ts;
+  GstFlowReturn ret = GST_FLOW_OK;
+  FluMp3Dec * dec = NULL;
+  GstClockTime new_ts;
+  gboolean discont;
+  
+  dec = FLUMP3DEC (gst_pad_get_parent (pad));
+  
+  discont = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT);
+  
+  /* We flush on disconts */
+  if (G_UNLIKELY (discont)) {
+    GST_DEBUG_OBJECT (dec, "this buffer has a DISCONT flag, flushing");
+    flump3dec_flush (dec);
+  }
   
   /* We've got a new buffer. Decode it! */
-
   new_ts = GST_BUFFER_TIMESTAMP (buf);
   GST_DEBUG ("New input buffer with TS %" G_GUINT64_FORMAT, new_ts);
 
   /* Give the buffer over to the decoder */
-  bs_add_buffer (flump3dec->bs, GST_BUFFER_DATA (buf), 
-      GST_BUFFER_SIZE (buf), buf, new_ts);
+  bs_add_buffer (dec->bs, GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf),
+      buf, new_ts);
 
-  return flump3dec_drain_avail (pad, TRUE);
+  ret = flump3dec_drain_avail (pad, TRUE);
+  
+  gst_object_unref (dec);
+  
+  return ret;
 }
 
 #define XING_FRAMES_FLAG     0x0001
@@ -333,7 +354,7 @@ gst_flump3dec_update_ts (FluMp3Dec *flump3dec, GstClockTime new_ts,
    * our current timestamp */
   if (GST_CLOCK_TIME_IS_VALID (out_ts)) {
     diff = ABS ((GstClockTimeDiff)(new_ts - out_ts));
-    if (diff > (frame_dur/2)) {
+    if ((GstClockTime) diff > (frame_dur / 2)) {
       GST_DEBUG_OBJECT (flump3dec, "Got frame with new TS %" 
           G_GUINT64_FORMAT " - using.", new_ts);
       out_ts = new_ts;
@@ -358,8 +379,7 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
 
   mp3tl_set_eos (flump3dec->dec, more_data);
   
-  while (bs_bits_avail (flump3dec->bs) > 0)
-  {
+  while (bs_bits_avail (flump3dec->bs) > 0) {
     /* Find an mp3 header */
     result = mp3tl_sync (flump3dec->dec);
     if (result != MP3TL_ERR_OK)
@@ -372,8 +392,8 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
       else if (result == MP3TL_ERR_STREAM)
         continue; /* Resync */
       else {
-	/* Fatal decoder error */
-	ret = GST_FLOW_ERROR;
+        /* Fatal decoder error */
+        ret = GST_FLOW_ERROR;
         goto decode_error;
       }
     }
@@ -394,17 +414,17 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
 
       if (mp3hdr->layer == 3) {
         codec = g_strdup_printf ("MPEG %d Audio, Layer %d (MP3)", 
-                    ver, mp3hdr->layer);
+            ver, mp3hdr->layer);
       }
       else {
         codec = g_strdup_printf ("MPEG %d Audio, Layer %d", 
-                    ver, mp3hdr->layer);
+            ver, mp3hdr->layer);
       }
       taglist = gst_tag_list_new();
       gst_tag_list_add(taglist, GST_TAG_MERGE_REPLACE, GST_TAG_AUDIO_CODEC,
-		       codec, NULL);
+          codec, NULL);
       gst_element_found_tags_for_pad(GST_ELEMENT(flump3dec), 
-				     flump3dec->srcpad, taglist);
+          flump3dec->srcpad, taglist);
       g_free (codec);
       /* end setting the tag */
 
@@ -419,7 +439,7 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
     flump3dec->frame_count++;
 
     /* Round the bitrate to the nearest kbps */
-    flump3dec->avg_bitrate = 
+    flump3dec->avg_bitrate = (guint)
         (flump3dec->bitrate_sum / flump3dec->frame_count + 500);
     flump3dec->avg_bitrate -= flump3dec->avg_bitrate % 1000;
 
@@ -438,7 +458,7 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
 	      flump3dec->rate, mp3hdr->sample_rate,
 	      flump3dec->channels, mp3hdr->channels);
       if (!gst_pad_set_caps (flump3dec->srcpad, caps)) {
-	gst_caps_unref (caps);
+        gst_caps_unref (caps);
         GST_ELEMENT_ERROR (flump3dec, CORE, NEGOTIATION, (NULL), (NULL));
 
         return GST_FLOW_ERROR;
@@ -461,8 +481,8 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
 
     /* We have enough bytes in the store, decode a frame */
     ret = gst_pad_alloc_buffer (flump3dec->srcpad, GST_BUFFER_OFFSET_NONE, 
-                mp3hdr->frame_samples * flump3dec->bytes_per_sample,
-                GST_PAD_CAPS (flump3dec->srcpad), &out_buf);
+        mp3hdr->frame_samples * flump3dec->bytes_per_sample,
+        GST_PAD_CAPS (flump3dec->srcpad), &out_buf);
     if (ret != GST_FLOW_OK) {
       GST_DEBUG_OBJECT (flump3dec, "Peer doesn't want buffer, skipping decode");
       /* Peer didn't want the buffer */
@@ -477,7 +497,7 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
     }
 
     result = mp3tl_decode_frame (flump3dec->dec, GST_BUFFER_DATA (out_buf),
-                 GST_BUFFER_SIZE (out_buf), &dec_ts);
+        GST_BUFFER_SIZE (out_buf), &dec_ts);
 
     if (result != MP3TL_ERR_OK) {
       /* Free up the buffer we allocated above */
@@ -493,26 +513,26 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
         /* Update time, and repeat the previous frame if we have one */
         flump3dec->bad = TRUE;
 
-	if (flump3dec->pending_frame != NULL) {
-	  GST_DEBUG_OBJECT (flump3dec, "Bad frame - using previous frame");
-	  out_buf = gst_buffer_create_sub (flump3dec->pending_frame, 0, 
-		        GST_BUFFER_SIZE (flump3dec->pending_frame));
-	  if (out_buf == NULL) {
-	    ret = GST_FLOW_ERROR;
-	    goto no_buffer;
-	  }
-	  gst_buffer_set_caps (out_buf, 
-	      GST_BUFFER_CAPS (flump3dec->pending_frame));
-	}
-	else {
-	  GST_DEBUG_OBJECT (flump3dec, "Bad frame - no existing frame. Skipping");
-	  flump3dec->next_ts += gst_util_uint64_scale (GST_SECOND, 
-                                    mp3hdr->frame_samples, mp3hdr->sample_rate);
-	  continue;
-	}
+      	if (flump3dec->pending_frame != NULL) {
+      	  GST_DEBUG_OBJECT (flump3dec, "Bad frame - using previous frame");
+      	  out_buf = gst_buffer_create_sub (flump3dec->pending_frame, 0, 
+              GST_BUFFER_SIZE (flump3dec->pending_frame));
+      	  if (out_buf == NULL) {
+      	    ret = GST_FLOW_ERROR;
+      	    goto no_buffer;
+      	  }
+      	  gst_buffer_set_caps (out_buf,
+              GST_BUFFER_CAPS (flump3dec->pending_frame));
+      	}
+      	else {
+      	  GST_DEBUG_OBJECT (flump3dec, "Bad frame - no existing frame. Skipping");
+      	  flump3dec->next_ts += gst_util_uint64_scale (GST_SECOND, 
+              mp3hdr->frame_samples, mp3hdr->sample_rate);
+      	  continue;
+      	}
       }
       else {
-	ret = GST_FLOW_ERROR;
+        ret = GST_FLOW_ERROR;
         goto decode_error;
       }
     }
@@ -521,15 +541,14 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
     flump3dec->bad = FALSE;
 
     /* Set the bitrate tag if changed (only care about changes over 10kbps) */
-    if ((flump3dec->last_posted_bitrate / 10240) != (flump3dec->avg_bitrate / 10240))
-      {
-	flump3dec->last_posted_bitrate = flump3dec->avg_bitrate;
-	taglist = gst_tag_list_new();
-	gst_tag_list_add(taglist, GST_TAG_MERGE_REPLACE, GST_TAG_BITRATE,
-			 flump3dec->last_posted_bitrate, NULL);
-	gst_element_found_tags_for_pad(GST_ELEMENT(flump3dec), 
-				       flump3dec->srcpad, taglist);
-      }
+    if ((flump3dec->last_posted_bitrate / 10240) != (flump3dec->avg_bitrate / 10240)) {
+      flump3dec->last_posted_bitrate = flump3dec->avg_bitrate;
+    	taglist = gst_tag_list_new();
+    	gst_tag_list_add(taglist, GST_TAG_MERGE_REPLACE, GST_TAG_BITRATE,
+          flump3dec->last_posted_bitrate, NULL);
+    	gst_element_found_tags_for_pad(GST_ELEMENT(flump3dec),  flump3dec->srcpad,
+          taglist);
+    }
     
 
     if (GST_CLOCK_TIME_IS_VALID (dec_ts) && 
@@ -539,10 +558,15 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
 
       flump3dec->last_dec_ts = dec_ts;
     }
+    
+    if (G_UNLIKELY (flump3dec->need_discont)) {
+      GST_BUFFER_FLAG_SET (out_buf, GST_BUFFER_FLAG_DISCONT);
+      flump3dec->need_discont = FALSE;
+    }
       
     GST_BUFFER_TIMESTAMP (out_buf) = flump3dec->next_ts;
     GST_BUFFER_DURATION (out_buf) = gst_util_uint64_scale (GST_SECOND, 
-                                    mp3hdr->frame_samples, mp3hdr->sample_rate);
+        mp3hdr->frame_samples, mp3hdr->sample_rate);
     flump3dec->next_ts += GST_BUFFER_DURATION (out_buf);
 
     GST_DEBUG_OBJECT (flump3dec, "Have new buffer, size %u, ts %" 
@@ -555,7 +579,7 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
     }
     else {
       /* push previous frame, queue current frame. */
-      GST_DEBUG_OBJECT (flump3dec, "Pushing previous frame, ts %"
+      GST_DEBUG_OBJECT (flump3dec, "Pushing previous frame, ts %" \
           GST_TIME_FORMAT, 
           GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (flump3dec->pending_frame))); 
 
@@ -615,12 +639,13 @@ flump3dec_sink_event (GstPad *pad, GstEvent *event)
       gboolean result, update;
       gboolean converted = FALSE;
 
+      flump3dec_drain_avail (pad, FALSE);
+
       gst_event_parse_new_segment (event, &update, &rate, &format,
           &start, &end, &base);
 
-      GST_DEBUG_OBJECT (flump3dec, 
-          "new segment, format=%d, base=%lld, start = %lld, end=%lld", format, base,
-	  start, end);
+      GST_DEBUG_OBJECT (flump3dec,  "new segment, format=%d, base=%lld, " \
+          "start = %lld, end=%lld", format, base, start, end);
 
       if (format == GST_FORMAT_BYTES) {
         gint64 disc_start, disc_end, disc_base;
@@ -628,19 +653,17 @@ flump3dec_sink_event (GstPad *pad, GstEvent *event)
         if (flump3_inbytes_to_time (flump3dec, start, &disc_start) &&
             flump3_inbytes_to_time (flump3dec, end, &disc_end) &&
             flump3_inbytes_to_time (flump3dec, base, &disc_base)) {
-	  gst_event_unref (event);
-	  event = gst_event_new_new_segment (FALSE, rate, GST_FORMAT_TIME,
-	      disc_start, disc_end, disc_base);
+          gst_event_unref (event);
+          event = gst_event_new_new_segment (FALSE, rate, GST_FORMAT_TIME,
+              disc_start, disc_end, disc_base);
           flump3dec->next_ts = disc_start;
 
-          GST_DEBUG_OBJECT (flump3dec, 
-              "Converted to TIME, base=%lld, start = %lld, end=%lld", disc_base,
-	      disc_start, disc_end);
-	  converted = TRUE;
-	}
-      } 
-      else if (format == GST_FORMAT_TIME)
-      {
+          GST_DEBUG_OBJECT (flump3dec,  "Converted to TIME, base=%lld, " \
+              "start = %lld, end=%lld", disc_base, disc_start, disc_end);
+          converted = TRUE;
+        }
+      }
+      else if (format == GST_FORMAT_TIME) {
         GST_DEBUG_OBJECT (flump3dec, "Got segment in time format");
         flump3dec->next_ts = start;
       }
@@ -649,7 +672,7 @@ flump3dec_sink_event (GstPad *pad, GstEvent *event)
         gst_event_unref (event);
         GST_DEBUG_OBJECT (flump3dec, "creating new time segment");
         event = gst_event_new_new_segment (FALSE, rate, GST_FORMAT_TIME,
-                    0, GST_CLOCK_TIME_NONE, 0);
+            0, GST_CLOCK_TIME_NONE, 0);
         flump3dec->next_ts = 0;
       }
 
@@ -668,7 +691,7 @@ flump3dec_sink_event (GstPad *pad, GstEvent *event)
       break;
   }
 
-  return gst_pad_event_default (pad, event);
+  return gst_pad_push_event (flump3dec->srcpad, event);
 }
 
 static GstStateChangeReturn 
@@ -822,7 +845,7 @@ static gboolean flump3dec_src_query (GstPad *pad, GstQuery *query)
       /* If the format is BYTES or SAMPLES (default), we'll calculate it from 
        * time, else let's see if upstream knows the duration in the right 
        * format */
-      if (format != GST_FORMAT_BYTES && format != GST_FORMAT_BYTES && 
+      if (format != GST_FORMAT_BYTES && format != GST_FORMAT_DEFAULT && 
           gst_pad_query (peer, query)) {
         gst_object_unref (peer);
         res = TRUE;
@@ -1085,8 +1108,8 @@ plugin_init (GstPlugin * plugin)
 /*
  * FIXME: Fill in the license, requires new enums in GStreamer
  */
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR, GST_VERSION_MINOR,
-  "flump3dec", "Fluendo MP3 decoder",
+FLUENDO_PLUGIN_DEFINE (GST_VERSION_MAJOR, GST_VERSION_MINOR,
+  "flump3dec", flump3dec, "Fluendo MP3 decoder",
   plugin_init, VERSION, GST_LICENSE_UNKNOWN, "Fluendo MP3 Decoder",
   "http://www.fluendo.com"
 )
